'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.mockSubscribeAdapter = exports.subscribe = undefined;

var subscribe = exports.subscribe = function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(store, subscribeAdapter, viewModels, subscribers, requests, action) {
        var viewModelName, aggregateId, needChange, key, rawState, state;
        return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        viewModelName = action.viewModelName, aggregateId = action.aggregateId;
                        needChange = !subscribers.viewModels[viewModelName] || !subscribers.aggregateIds[aggregateId];


                        subscribers.viewModels[viewModelName] = (subscribers.viewModels[viewModelName] || 0) + 1;
                        subscribers.aggregateIds[aggregateId] = (subscribers.aggregateIds[aggregateId] || 0) + 1;

                        if (!needChange) {
                            _context.next = 12;
                            break;
                        }

                        key = (0, _util.getKey)(viewModelName, aggregateId);

                        requests[key] = true;

                        _context.next = 9;
                        return (0, _load_initial_state2.default)(viewModelName, aggregateId);

                    case 9:
                        rawState = _context.sent;
                        state = viewModels.find(function (_ref3) {
                            var name = _ref3.name;
                            return name === viewModelName;
                        }).deserializeState(rawState);


                        if (requests[key]) {
                            delete requests[key];

                            store.dispatch(_actions2.default.merge(viewModelName, aggregateId, state));

                            subscribeAdapter.setSubscription({
                                types: getEventTypes(viewModels, subscribers),
                                aggregateIds: getAggregateIds(viewModels, subscribers)
                            });
                        }

                    case 12:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    return function subscribe(_x, _x2, _x3, _x4, _x5, _x6) {
        return _ref2.apply(this, arguments);
    };
}();

exports.getEventTypes = getEventTypes;
exports.getAggregateIds = getAggregateIds;
exports.unsubscribe = unsubscribe;
exports.createResolveMiddleware = createResolveMiddleware;

var _action_types = require('./action_types');

var _actions = require('./actions');

var _actions2 = _interopRequireDefault(_actions);

var _subscribe_adapter = require('./subscribe_adapter');

var _subscribe_adapter2 = _interopRequireDefault(_subscribe_adapter);

var _send_command = require('./send_command');

var _send_command2 = _interopRequireDefault(_send_command);

var _load_initial_state = require('./load_initial_state');

var _load_initial_state2 = _interopRequireDefault(_load_initial_state);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var REFRESH_TIMEOUT = 1000;

function getEventTypes(viewModels, subscribers) {
    var eventTypes = {};

    Object.keys(subscribers.viewModels).forEach(function (viewModelName) {
        if (!subscribers.viewModels[viewModelName]) {
            return;
        }

        var _viewModels$find$proj = viewModels.find(function (_ref) {
            var name = _ref.name;
            return name === viewModelName;
        }).projection,
            Init = _viewModels$find$proj.Init,
            projection = _objectWithoutProperties(_viewModels$find$proj, ['Init']);

        Object.keys(projection).forEach(function (eventType) {
            eventTypes[eventType] = true;
        });
    });

    return Object.keys(eventTypes);
}

function getAggregateIds(viewModels, subscribers) {
    if (subscribers.aggregateIds['*'] > 0) {
        return '*';
    }

    return Object.keys(subscribers.aggregateIds).filter(function (aggregateId) {
        return subscribers.aggregateIds[aggregateId];
    });
}

function unsubscribe(store, subscribeAdapter, viewModels, subscribers, requests, action) {
    var viewModelName = action.viewModelName,
        aggregateId = action.aggregateId;


    subscribers.viewModels[viewModelName] = Math.max((subscribers.viewModels[viewModelName] || 0) - 1, 0);
    subscribers.aggregateIds[aggregateId] = Math.max((subscribers.aggregateIds[aggregateId] || 0) - 1, 0);

    var needChange = !subscribers.viewModels[viewModelName] || !subscribers.aggregateIds[aggregateId];

    var key = (0, _util.getKey)(viewModelName, aggregateId);
    delete requests[key];

    if (needChange) {
        subscribeAdapter.setSubscription({
            types: getEventTypes(viewModels, subscribers),
            aggregateIds: getAggregateIds(viewModels, subscribers)
        });
    }
}

var isClient = typeof window !== 'undefined';

var mockSubscribeAdapter = exports.mockSubscribeAdapter = {
    onEvent: function onEvent() {},
    onDisconnect: function onDisconnect() {},
    setSubscription: function setSubscription() {}
};

function createResolveMiddleware(_ref4) {
    var viewModels = _ref4.viewModels,
        _ref4$subscribeAdapte = _ref4.subscribeAdapter,
        subscribeAdapter = _ref4$subscribeAdapte === undefined ? _subscribe_adapter2.default : _ref4$subscribeAdapte;

    var subscribers = {
        viewModels: {},
        aggregateIds: {}
    };

    var requests = {};
    var loading = viewModels.reduce(function (acc, _ref5) {
        var name = _ref5.name;

        acc[name] = {};
        return acc;
    }, {});

    return function (store) {
        Object.defineProperty(store.getState, 'isLoadingViewModel', {
            enumerable: false,
            configurable: false,
            writable: false,
            value: function value(viewModelName, aggregateId) {
                return !!loading[viewModelName][aggregateId];
            }
        });

        store.dispatch(_actions2.default.provideViewModels(viewModels));

        var adapter = isClient ? subscribeAdapter() : mockSubscribeAdapter;
        adapter.onEvent(function (event) {
            return store.dispatch(event);
        });
        adapter.onDisconnect(function (error) {
            return store.dispatch(_actions2.default.disconnect(error));
        });

        return function (next) {
            return function (action) {
                switch (action.type) {
                    case _action_types.SUBSCRIBE:
                        {
                            var viewModelName = action.viewModelName,
                                aggregateId = action.aggregateId;

                            loading[viewModelName][aggregateId] = true;

                            if (isClient) {
                                subscribe(store, adapter, viewModels, subscribers, requests, action).catch(function (error) {
                                    return setTimeout(function () {
                                        // eslint-disable-next-line no-console
                                        console.error(error);
                                        store.dispatch(action);
                                    }, REFRESH_TIMEOUT);
                                });
                            }

                            break;
                        }
                    case _action_types.UNSUBSCRIBE:
                        {
                            var _viewModelName = action.viewModelName,
                                _aggregateId = action.aggregateId;

                            delete loading[_viewModelName][_aggregateId];

                            if (isClient) {
                                unsubscribe(store, adapter, viewModels, subscribers, requests, action);
                            }

                            break;
                        }
                    case _action_types.SEND_COMMAND:
                        {
                            if (isClient) {
                                (0, _send_command2.default)(store, action);
                            }

                            break;
                        }
                    default:
                }

                return next(action);
            };
        };
    };
}

exports.default = createResolveMiddleware;