'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.subscribeHandler = subscribeHandler;
exports.unsubscribeHandler = unsubscribeHandler;
exports.mergeHandler = mergeHandler;
exports.provideViewModelsHandler = provideViewModelsHandler;
exports.createMap = createMap;
exports.viewModelEventHandler = viewModelEventHandler;
exports.default = createViewModelsReducer;

var _action_types = require('./action_types');

var _util = require('./util');

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function subscribeHandler(_ref, state, _ref2) {
    var subscribers = _ref.subscribers,
        viewModels = _ref.viewModels;
    var viewModelName = _ref2.viewModelName,
        aggregateId = _ref2.aggregateId;

    var key = (0, _util.getKey)(viewModelName, aggregateId);

    if (subscribers[key]) {
        subscribers[key]++;
        return state;
    }

    subscribers[key] = 1;

    return _extends({}, state, _defineProperty({}, viewModelName, _extends({}, state[viewModelName], _defineProperty({}, aggregateId, (viewModels.find(function (_ref3) {
        var name = _ref3.name;
        return viewModelName === name;
    }).projection.Init || function () {})()))));
}

function unsubscribeHandler(_ref4, state, _ref5) {
    var subscribers = _ref4.subscribers;
    var viewModelName = _ref5.viewModelName,
        aggregateId = _ref5.aggregateId;

    var key = (0, _util.getKey)(viewModelName, aggregateId);

    if (subscribers[key] > 1) {
        subscribers[key]--;
        return state;
    }

    subscribers[key] = 0;

    var _state$viewModelName = state[viewModelName],
        _ = _state$viewModelName[aggregateId],
        nextViewModel = _objectWithoutProperties(_state$viewModelName, [aggregateId]);

    return _extends({}, state, _defineProperty({}, viewModelName, nextViewModel));
}

function mergeHandler(_, state, _ref6) {
    var viewModelName = _ref6.viewModelName,
        aggregateId = _ref6.aggregateId,
        actionState = _ref6.state;

    return _extends({}, state, _defineProperty({}, viewModelName, _extends({}, state[viewModelName], _defineProperty({}, aggregateId, actionState))));
}

function provideViewModelsHandler(context, state, _ref7) {
    var viewModels = _ref7.viewModels;
    var handlers = context.handlers,
        initialState = context.initialState;

    context.viewModels = viewModels;

    delete handlers[_action_types.PROVIDE_VIEW_MODELS];

    handlers[_action_types.SUBSCRIBE] = subscribeHandler.bind(null, context);

    handlers[_action_types.UNSUBSCRIBE] = unsubscribeHandler.bind(null, context);

    handlers[_action_types.MERGE] = mergeHandler.bind(null, context);

    viewModels.forEach(function (_ref8) {
        var viewModelName = _ref8.name;

        initialState[viewModelName] = _extends({}, initialState[viewModelName]);
    });

    var map = createMap(viewModels);

    Object.keys(map).forEach(function (eventType) {
        handlers[eventType] = viewModelEventHandler.bind(null, map[eventType]);
    });

    return initialState;
}

function createMap(viewModels) {
    return viewModels.reduce(function (acc, _ref9) {
        var viewModelName = _ref9.name,
            _ref9$projection = _ref9.projection,
            Init = _ref9$projection.Init,
            projection = _objectWithoutProperties(_ref9$projection, ['Init']);

        Object.keys(projection).forEach(function (eventType) {
            if (!acc[eventType]) {
                acc[eventType] = {};
            }

            acc[eventType][viewModelName] = projection[eventType];
        });
        return acc;
    }, {});
}

function viewModelEventHandler(viewModels, state, action) {
    var nextState = _extends({}, state);

    Object.keys(viewModels).forEach(function (viewModelName) {
        if (!state[viewModelName]) return;

        if (state[viewModelName].hasOwnProperty('*')) {
            var _viewModelState = state[viewModelName]['*'];

            var _result = viewModels[viewModelName](_viewModelState, action);

            nextState[viewModelName] = _extends({}, nextState[viewModelName], {
                '*': _result
            });
        }

        if (!state[viewModelName].hasOwnProperty(action.aggregateId)) return;

        var viewModelState = state[viewModelName][action.aggregateId];

        var result = viewModels[viewModelName](viewModelState, action);

        nextState[viewModelName] = _extends({}, nextState[viewModelName], _defineProperty({}, action.aggregateId, result));
    });

    return nextState;
}

function createViewModelsReducer() {
    var context = {
        initialState: {},
        handlers: {},
        subscribers: {}
    };

    context.handlers[_action_types.PROVIDE_VIEW_MODELS] = provideViewModelsHandler.bind(null, context);

    return function () {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var action = arguments[1];

        var eventHandler = context.handlers[action.type];

        if (eventHandler) {
            return eventHandler(state, action);
        }

        return state;
    };
}